"""
Extension for the `test` cog.
Loading this file via `commands.Bot.load_extension` will add the `test` cog to the bot.
"""

import logging

import textwrap
from markdownify import markdownify
from discord import app_commands, Interaction, Embed
from discord.ext import commands, tasks
from sqlalchemy import insert, select, and_

from db import DatabaseManager, AuditModel, SentArticleModel, RssSourceModel
from feed import Feeds, get_source

log = logging.getLogger(__name__)


class Test(commands.Cog):
    """
    News cog.
    Delivers embeds of news articles to discord channels.
    """

    def __init__(self, bot):
        super().__init__()
        self.bot = bot

    @commands.Cog.listener()
    async def on_ready(self):
        log.info(f"{self.__class__.__name__} cog is ready")

    async def source_autocomplete(self, inter: Interaction, current: str):
        """

        """

        async with DatabaseManager() as database:
            whereclause = and_(
                RssSourceModel.discord_server_id == inter.guild_id,
                RssSourceModel.rss_url.ilike(f"%{current}%")
            )
            query = select(RssSourceModel).where(whereclause)
            result = await database.session.execute(query)
            sources = [
                app_commands.Choice(name=rss.rss_url, value=rss.rss_url)
                for rss in result.scalars().all()
            ]

        return sources

    @app_commands.command(name="test-latest-article")
    @app_commands.autocomplete(source=source_autocomplete)
    async def test_news(self, inter: Interaction, source: str):

        await inter.response.defer()
        await self.bot.audit("Requesting latest article.", inter.user.id)

        try:
            source = get_source(source)
            article = source.get_latest_article()
        except IndexError as e:
            log.error(e)
            await inter.followup.send("An error occured, it's possible that the source provided was bad.")
            return

        md_description = markdownify(article.description, strip=("img",))
        article_description = textwrap.shorten(md_description, 4096)

        embed = Embed(
            title=article.title,
            description=article_description,
            url=article.url,
            timestamp=article.published,
        )
        embed.set_thumbnail(url=source.icon_url)
        embed.set_image(url=await article.get_thumbnail_url())
        embed.set_footer(text=article.author)
        embed.set_author(
            name=source.name,
            url=source.url,
        )

        async with DatabaseManager() as database:
            query = insert(SentArticleModel).values(
                discord_server_id=inter.guild_id,
                discord_channel_id=inter.channel_id,
                discord_message_id=inter.id,
                article_url=article.url
            )
            await database.session.execute(query)

        await inter.followup.send(embed=embed)


async def setup(bot):
    """
    Setup function for this extension.
    Adds the `ErrorCog` cog to the bot.
    """

    cog = Test(bot)
    await bot.add_cog(cog)
    log.info(f"Added {cog.__class__.__name__} cog")
